"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ConversionTable_1 = __importDefault(require("./ConversionTable"));
// Regex to extract quantity and type of base measurement
// Expects `<quantity> <type>` - e.g. 12.5 cups
// Important capture groups: (1-based index)
//  1: Decimal or integer quantity (e.g. 12.5)
//  3: Singular measurement type (e.g. 'cup')
var BASE_MEASUREMENT_TEST = /^([0-9]+([,.][0-9]+)?)\s([a-zA-Z]+?)(s\b|\b)/;
// Generate a new Error object when an invalid input is provided
var InvalidQuantityInputError = function () {
    return new Error("The input measurement must be in the format `<quantity> <unit>` where quantity is an integer or decimal.");
};
var Quantity = /** @class */ (function () {
    function Quantity(baseMeasurement) {
        // Input measurement must conform to the regex described above
        if (!BASE_MEASUREMENT_TEST.test(baseMeasurement)) {
            throw InvalidQuantityInputError();
        }
        // Execute regex on the lowercase input
        var measurementFields = BASE_MEASUREMENT_TEST.exec(baseMeasurement.toLowerCase());
        // If the regex failed, throw an error
        if (!measurementFields) {
            throw InvalidQuantityInputError();
        }
        // Store the raw input along with extracted quantity and units
        this.baseMeasurement = {
            raw: baseMeasurement,
            quantity: Number(measurementFields[1]),
            unit: String(measurementFields[3]),
        };
    }
    Quantity.prototype.toOunces = function () {
        return this.convertTo("ounce").toNumber();
    };
    Quantity.prototype.toCups = function () {
        return this.convertTo("cup").toNumber();
    };
    Quantity.prototype.toTablespoons = function () {
        return this.convertTo("tablespoon").toNumber();
    };
    Quantity.prototype.toTeaspoons = function () {
        return this.convertTo("teaspoon").toNumber();
    };
    Quantity.prototype.toPints = function () {
        return this.convertTo("pint").toNumber();
    };
    Quantity.prototype.toQuarts = function () {
        return this.convertTo("quart").toNumber();
    };
    Quantity.prototype.toGallons = function () {
        return this.convertTo("gallon").toNumber();
    };
    Quantity.prototype.convertTo = function (toUnit) {
        return new Quantity("".concat(ConversionTable_1.default.getConversionFactor(this.baseMeasurement.unit, toUnit) *
            this.baseMeasurement.quantity, " ").concat(toUnit));
    };
    Quantity.prototype.toNumber = function () {
        return Number(this.baseMeasurement.quantity);
    };
    Quantity.prototype.toString = function () {
        // Pluralize the unit if necessary
        // TODO: Determine if pluralization makes sense. E.g. if the unit is 'oz,' it doesn't make
        // sense to add an 's' even if the quantity is greater than 1.
        var unitString = this.baseMeasurement.quantity > 1
            ? "".concat(this.baseMeasurement.unit, "s")
            : this.baseMeasurement.unit;
        return "".concat(this.baseMeasurement.quantity, " ").concat(unitString);
    };
    return Quantity;
}());
exports.default = Quantity;
